@@ -80,7 +80,7 @@ couchTests.cookie_auth = function(debug) {
       T(usersDb.save(jasonUserDoc).ok);
 
       var checkDoc = open_as(usersDb, jasonUserDoc._id, "jan");
-      T(checkDoc.name == "Jason Davies");
+      TEquals("Jason Davies", checkDoc.name);
 
       var jchrisUserDoc = CouchDB.prepareUserDoc({
         name: "jchris@apache.org"
@@ -96,8 +96,8 @@ couchTests.cookie_auth = function(debug) {
         usersDb.save(duplicateJchrisDoc);
         T(false && "Can't create duplicate user names. Should have thrown an error.");
       } catch (e) {
-        T(e.error == "conflict");
-        T(usersDb.last_req.status == 409);
+        TEquals("conflict", e.error);
+        TEquals(409, usersDb.last_req.status);
       }
 
       // we can't create _names
@@ -109,8 +109,8 @@ couchTests.cookie_auth = function(debug) {
         usersDb.save(underscoreUserDoc);
         T(false && "Can't create underscore user names. Should have thrown an error.");
       } catch (e) {
-        T(e.error == "forbidden");
-        T(usersDb.last_req.status == 403);
+        TEquals("forbidden", e.error);
+        TEquals(403, usersDb.last_req.status);
       }
 
       // we can't create docs with malformed ids
@@ -124,13 +124,13 @@ couchTests.cookie_auth = function(debug) {
         usersDb.save(badIdDoc);
         T(false && "Can't create malformed docids. Should have thrown an error.");
       } catch (e) {
-        T(e.error == "forbidden");
-        T(usersDb.last_req.status == 403);
+        TEquals("forbidden", e.error);
+        TEquals(403, usersDb.last_req.status);
       }
 
       // login works
       T(CouchDB.login('Jason Davies', password).ok);
-      T(CouchDB.session().userCtx.name == 'Jason Davies');
+      TEquals('Jason Davies', CouchDB.session().userCtx.name);
 
       // JSON login works
       var xhr = CouchDB.request("POST", "/_session", {
@@ -142,7 +142,7 @@ couchTests.cookie_auth = function(debug) {
       });
 
       T(JSON.parse(xhr.responseText).ok);
-      T(CouchDB.session().userCtx.name == 'Jason Davies');
+      TEquals('Jason Davies', CouchDB.session().userCtx.name);
 
       // update one's own credentials document
       jasonUserDoc.foo=2;
@@ -153,8 +153,8 @@ couchTests.cookie_auth = function(debug) {
         usersDb.deleteDoc(jchrisUserDoc);
         T(false && "Can't delete other users docs. Should have thrown an error.");
       } catch (e) {
-        T(e.error == "forbidden");
-        T(usersDb.last_req.status == 403);
+        TEquals("forbidden", e.error);
+        TEquals(403, usersDb.last_req.status);
       }
 
       // TODO should login() throw an exception here?
@@ -164,21 +164,24 @@ couchTests.cookie_auth = function(debug) {
        // a failed login attempt should log you out
        T(CouchDB.session().userCtx.name != 'Jason Davies');
 
-       // test redirect
+       // test redirect on success
        xhr = CouchDB.request("POST", "/_session?next=/", {
          headers: {"Content-Type": "application/x-www-form-urlencoded"},
          body: "name=Jason%20Davies&password="+encodeURIComponent(password)
        });
-       // should this be a redirect code instead of 200?
-       // The cURL adapter is returning the expected 302 here.
-       // I imagine this has to do with whether the client is willing
-       // to follow the redirect, ie, the browser follows and does a
-       // GET on the returned Location
+       // the browser should transparently follow the redirect and GET the server root (/)
+       // see http://dev.w3.org/2006/webapi/XMLHttpRequest/#infrastructure-for-the-send-method
+       if (xhr.status == 200) {
+         T(/Welcome/.test(xhr.responseText))
+       }
+
+       // test redirect on fail
+       xhr = CouchDB.request("POST", "/_session?fail=/", {
+         headers: {"Content-Type": "application/x-www-form-urlencoded"},
+         body: "name=Jason%20Davies&password=foobar"
+       });
        if (xhr.status == 200) {
          T(/Welcome/.test(xhr.responseText));
-       } else {
-         T(xhr.status == 302);
-         T(xhr.getResponseHeader("Location"));
        }
 
       // test users db validations
@@ -275,6 +278,8 @@ couchTests.cookie_auth = function(debug) {
     [
      {section: "couch_httpd_auth",
       key: "authentication_db", value: "test_suite_users"},
+     {section: "couch_httpd_auth",
+      key: "iterations", value: "1"},
      {section: "admins",
        key: "jan", value: "apple"}
     ],
