@@ -16,15 +16,17 @@
 #include <jsapi.h>
 
 typedef struct {
-    int         use_http;
-    int         stack_size;
-    const char* script_name;
-    char*       script;
+    int          use_http;
+    int          stack_size;
+    const char** scripts;
+    const char*  uri_file;
+    JSString*    uri;
 } couch_args;
 
 couch_args* couch_parse_args(int argc, const char* argv[]);
 int couch_fgets(char* buf, int size, FILE* fp);
 JSString* couch_readline(JSContext* cx, FILE* fp);
+JSString* couch_readfile(JSContext* cx, const char* filename);
 void couch_print(JSContext* cx, uintN argc, jsval* argv);
 void couch_error(JSContext* cx, const char* mesg, JSErrorReport* report);
 JSBool couch_load_funcs(JSContext* cx, JSObject* obj, JSFunctionSpec* funcs);
